import json

from django.conf import settings
from django.http.multipartparser import MultiPartParser as DjangoMultiPartParser, MultiPartParserError
from humps import camelize, decamelize
from rest_framework.exceptions import ParseError
from rest_framework.parsers import FormParser, MultiPartParser, DataAndFiles, JSONParser


class CaseJSONParser(JSONParser):
    def parse(self, stream, media_type=None, parser_context=None):
        parser_context = parser_context or {}
        encoding = parser_context.get('encoding', settings.DEFAULT_CHARSET)

        try:
            data = stream.read().decode(encoding)
            return decamelize(json.loads(data))
        except ValueError as err:
            raise ParseError(f'JSON parse error - {err}')


# Parser for form data
class CaseFormParser(FormParser):
    def parse(self, stream, media_type=None, parser_context=None):
        return decamelize(super().parse(stream, media_type, parser_context))


# Parser for multipart form data, which may include file data
class CaseMultiPartParser(MultiPartParser):
    media_type = 'multipart/form-data'

    def parse(self, stream, media_type=None, parser_context=None):
        '''
        Parses the incoming bytestream as a multipart encoded form,
        and returns a DataAndFiles object.
        `.data` will be a `QueryDict` containing all the form parameters.
        `.files` will be a `QueryDict` containing all the form files.
        '''
        parser_context = parser_context or {}
        request = parser_context['request']
        encoding = parser_context.get('encoding', settings.DEFAULT_CHARSET)
        meta = request.META.copy()
        meta['CONTENT_TYPE'] = media_type
        upload_handlers = request.upload_handlers

        try:
            parser = DjangoMultiPartParser(meta, stream, upload_handlers, encoding)
            data, files = parser.parse()

            return DataAndFiles(decamelize(data), decamelize(files), )
        except MultiPartParserError as err:
            raise ParseError(f'Multipart form parse error - {err}')



from humps import camelize
from rest_framework.renderers import BrowsableAPIRenderer, JSONRenderer


class CaseJSONRenderer(JSONRenderer):
    def render(self, data, *args, **kwargs):
        return super().render(camelize(data), *args, **kwargs)


class CaseBrowsableAPIRenderer(BrowsableAPIRenderer):
    def render(self, data, *args, **kwargs):
        return super(CaseBrowsableAPIRenderer, self).render(camelize(data), *args, **kwargs)



from humps import camelize, decamelize


class CaseMiddleware:
    def __init__(self, get_response):
         self.get_response = get_response

    def __call__(self, request):
        request.GET = decamelize(request.GET)
        response = self.get_response(request)

        return response



import os
import sys


try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

readme = open('README.rst').read()
import drf_case_middleware

setup(
    name='drf-case-middleware',
    version=drf_case_middleware.__version__,
    description='Camel case to snake case and snake case to camel case for Django REST framework',
    author='Jiyoon Ha',
    author_email='punkkid001@gmail.com',
    url='https://github.com/django-breaker/drf-case-middleware',
    packages=['drf_case_middleware'],
    package_dir={'drf_case_middleware': 'drf_case_middleware'},
    include_package_data=True,
    python_requires='>=3.6',
    install_requires=[],
    license='MIT',
    zip_safe=False,
    keywords='drf-case-middleware',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
)
